import TwitchEventSub.event_types.event_types as event_types

EVENT_VERSIONS = {
    event_types.EventType.CHANNEL_UPDATE: "1",
    event_types.EventType.CHANNEL_FOLLOW: "1",
    event_types.EventType.CHANNEL_SUBSCRIBE: "1",
    event_types.EventType.CHANNEL_SUBSCRIPTION_END: "1",
    event_types.EventType.CHANNEL_SUBSCRIPTION_GIFT: "1",
    event_types.EventType.CHANNEL_SUBSCRIPTION_MESSAGE: "1",
    event_types.EventType.CHANNEL_CHEER: "1",
    event_types.EventType.CHANNEL_RAID: "1",
    event_types.EventType.CHANNEL_BAN: "1",
    event_types.EventType.CHANNEL_UNBAN: "1",
    event_types.EventType.CHANNEL_MODERATOR_ADD: "1",
    event_types.EventType.CHANNEL_MODERATOR_REMOVE: "1",
    event_types.EventType.CHANNEL_POINTS_CUSTOM_REWARD_ADD: "1",
    event_types.EventType.CHANNEL_POINTS_CUSTOM_REWARD_UPDATE: "1",
    event_types.EventType.CHANNEL_POINTS_CUSTOM_REWARD_REMOVE: "1",
    event_types.EventType.CHANNEL_POINTS_CUSTOM_REWARD_REDEMPTION_ADD: "1",
    event_types.EventType.CHANNEL_POINTS_CUSTOM_REWARD_REDEMPTION_UPDATE: "1",
    event_types.EventType.CHANNEL_POLL_BEGIN: "1",
    event_types.EventType.CHANNEL_POLL_PROGRESS: "1",
    event_types.EventType.CHANNEL_POLL_END: "1",
    event_types.EventType.CHANNEL_PREDICTION_BEGIN: "1",
    event_types.EventType.CHANNEL_PREDICTION_PROGRESS: "1",
    event_types.EventType.CHANNEL_PREDICTION_LOCK: "1",
    event_types.EventType.CHANNEL_PREDICTION_END: "1",
    event_types.EventType.CHARITY_DONATION: "beta",
    event_types.EventType.CHARITY_CAMPAIGN_START: "beta",
    event_types.EventType.CHARITY_CAMPAIGN_PROGRESS: "beta",
    event_types.EventType.CHARITY_CAMPAIGN_STOP: "beta",
    event_types.EventType.DROP_ENTITLEMENT_GRANT: "1",
    event_types.EventType.EXTENSION_BITS_TRANSACTION_CREATE: "1",
    event_types.EventType.GOAL_BEGIN: "1",
    event_types.EventType.GOAL_PROGRESS: "1",
    event_types.EventType.GOAL_END: "1",
    event_types.EventType.HYPE_TRAIN_BEGIN: "1",
    event_types.EventType.HYPE_TRAIN_PROGRESS: "1",
    event_types.EventType.HYPE_TRAIN_END: "1",
    event_types.EventType.SHIELD_MODE_BEGIN: "beta",
    event_types.EventType.SHIELD_MODE_END: "beta",
    event_types.EventType.STREAM_ONLINE: "1",
    event_types.EventType.STREAM_OFFLINE: "1",
    event_types.EventType.USER_AUTHORIZATION_GRANT: "1",
    event_types.EventType.USER_AUTHORIZATION_REVOKE: "1",
    event_types.EventType.USER_UPDATE: "1"
}
