import TwitchEventSub.event_types.event_types as event_types

EVENT_SCOPES = {
    event_types.EventType.CHANNEL_UPDATE: None,
    event_types.EventType.CHANNEL_FOLLOW: None,
    event_types.EventType.CHANNEL_SUBSCRIBE: "channel:read:subscriptions",
    event_types.EventType.CHANNEL_SUBSCRIPTION_END: "channel:read:subscriptions",
    event_types.EventType.CHANNEL_SUBSCRIPTION_GIFT: "channel:read:subscriptions",
    event_types.EventType.CHANNEL_SUBSCRIPTION_MESSAGE: "channel:read:subscriptions",
    event_types.EventType.CHANNEL_CHEER: "bits:read",
    event_types.EventType.CHANNEL_RAID: None,
    event_types.EventType.CHANNEL_BAN: "channel:moderate",
    event_types.EventType.CHANNEL_UNBAN: "channel:moderate",
    event_types.EventType.CHANNEL_MODERATOR_ADD: "moderation:read",
    event_types.EventType.CHANNEL_MODERATOR_REMOVE: "moderation:read",
    event_types.EventType.CHANNEL_POINTS_CUSTOM_REWARD_ADD: "channel:read:redemptions",
    event_types.EventType.CHANNEL_POINTS_CUSTOM_REWARD_UPDATE: "channel:read:redemptions",
    event_types.EventType.CHANNEL_POINTS_CUSTOM_REWARD_REMOVE: "channel:read:redemptions",
    event_types.EventType.CHANNEL_POINTS_CUSTOM_REWARD_REDEMPTION_ADD: "channel:read:redemptions",
    event_types.EventType.CHANNEL_POINTS_CUSTOM_REWARD_REDEMPTION_UPDATE: "channel:read:redemptions",
    event_types.EventType.CHANNEL_POLL_BEGIN: "channel:read:polls",
    event_types.EventType.CHANNEL_POLL_PROGRESS: "channel:read:polls",
    event_types.EventType.CHANNEL_POLL_END: "channel:read:polls",
    event_types.EventType.CHANNEL_PREDICTION_BEGIN: "channel:read:predictions",
    event_types.EventType.CHANNEL_PREDICTION_PROGRESS: "channel:read:predictions",
    event_types.EventType.CHANNEL_PREDICTION_LOCK: "channel:read:predictions",
    event_types.EventType.CHANNEL_PREDICTION_END: "channel:read:predictions",
    event_types.EventType.CHARITY_DONATION: "channel:read:charity",
    event_types.EventType.CHARITY_CAMPAIGN_START: "channel:read:charity",
    event_types.EventType.CHARITY_CAMPAIGN_PROGRESS: "channel:read:charity",
    event_types.EventType.CHARITY_CAMPAIGN_STOP: "channel:read:charity",
    event_types.EventType.DROP_ENTITLEMENT_GRANT: "drop.entitlement.grant",
    event_types.EventType.EXTENSION_BITS_TRANSACTION_CREATE: None,
    event_types.EventType.GOAL_BEGIN: "channel:read:goals",
    event_types.EventType.GOAL_PROGRESS: "channel:read:goals",
    event_types.EventType.GOAL_END: "channel:read:goals",
    event_types.EventType.HYPE_TRAIN_BEGIN: "channel:read:hype_train",
    event_types.EventType.HYPE_TRAIN_PROGRESS: "channel:read:hype_train",
    event_types.EventType.HYPE_TRAIN_END: "channel:read:hype_train",
    event_types.EventType.SHIELD_MODE_BEGIN: "moderator:read:shield_mode",
    event_types.EventType.SHIELD_MODE_END: "moderator:read:shield_mode",
    event_types.EventType.STREAM_ONLINE: None,
    event_types.EventType.STREAM_OFFLINE: None,
    event_types.EventType.USER_AUTHORIZATION_GRANT: None,
    event_types.EventType.USER_AUTHORIZATION_REVOKE: None,
    event_types.EventType.USER_UPDATE: None
}
